######################################################################
########## 2. MUTATIONAL MODELS AND SEQUENCE EVOLUTION ###############
######################################################################
  comment <- c("
  How many mutations we expect between two lineages?
  We need to know the time to the ancestral sequence and the mutation rate (2*u*t) under neutrality.
  We assume the substitutiions accumulate constantly across the time. (molecular clock).

  The time to the common ancestor is determined by: the time of speciation
  and the population size of the ancestor population. The split time between lineages may not be 
  coincident with the common ancestor, because this one will be some time (in relation to ancestral population size) 
  before to the split of the lineages.
  Note that some of the mutations are polymorphic in the current population (recent mutations) and 
  some are substitutions (fixed mutations) between lineages.
  ")
  
  ###################################################################
  # 2.1 DNA sequence has ”only” four different states (nucleotides) #
  ###################################################################
  comment <- c("Although we assume this simple model, we have to consider the fact that a DNA sequence contains four different nucleotides (A,C,G,T). In a finite length sequence, the mutations can accumulate over the same positions, and the mutations may revert to the ancestral state. Let’s simulate a sequence first:")
    
  #The problem of having four nucleotides and finite length. Let's generate a sequence:'
  Tnt <- c("A","C","G","T")
  L <- 100
  seq <- sample(x=Tnt,size=L,replace=TRUE,prob=c(0.25,0.25,0.25,0.25))
  seq

  #How many SUBSTITUTIONS we can see and how many substitutions really happened between 2 sequences that diverged G generations ago?
  seq.ancestral <- seq
  subst.rate = mu <- 1e-6  #assumption under neutrality: rate of substitutions is equal to mutation rate

  G <- 1e6
  #N <- 2
  n.subst.lineage.1 <- rpois(n=1,lambda=subst.rate * L * G)
  n.subst.lineage.1
  n.subst.lineage.2 <- rpois(n=1,lambda=subst.rate * L * G)
  n.subst.lineage.2
  #Total mutations between the two sequences
  n.subst.lineage.1 + n.subst.lineage.2
  
  ###################################################################
  # 2.2 The Jukes and Cantor model for correction of substitutions ##
  ###################################################################
  #what is the probability that a given nucleotide (ex. A) mutate to another (ex. C)?
  #we use a rate matrix for all combinations of mutations: 
  Q <- matrix(1/3,ncol=4,nrow=4,dimnames=list(c("A","C","G","T"),c("A","C","G","T")))
  for(x in 1:4) {Q[x,x] <- 0; Q[x,x] <- -sum(Q[x,])}
  Q #Substitution Rate matrix
  #This matrix shows the most simple scenario (Jukes and Cantor model): 
  #all nucleotides change with the same probability
  
  ###################################################################
  ######## 2.2.1 The number of differences under the JC model #######
  ###################################################################
  comment <- c("Let’s calculate how many differences we really observe when compare two sequences assuming the Jukes and Cantor Model. We construct first a function:")
  
  #The number of mutations accumulate over the sequence:
  substitutions.on.sequence <- function(seq,substitutions,Q,Tnt) {
  #seq is a vector of symbols (eg., A, T, A, G, C ...etc.)
  #substitutions is the total number of substitutions
  #Q is the substitution rate matrix (C x C)
  #Tnt is a vector of C values containing the possible symbols in the seq, sorted as in Q (ex. A, C, G, T)
    seq1 <- seq
    len <- length(seq)
    position.substitution <- sample(c(1:len),size=substitutions,replace=TRUE) #where the substitutions fall on seq
    number.symbols <- c(1:length(Tnt))
    
    for(ps in position.substitution) {
      nt <- which(Tnt==seq1[ps]) #which variant is in seq[ps]
      symbol <- number.symbols[-nt] #count the possible variants (not the original)
      seq1[ps] <- sample(Tnt[-nt],1,prob=Q[nt,-nt]) #assign the new nucleotide
    }   
    seq1 #the output is the new vector of symbols after including substitutions
  }
  
  #The number of differences observed can be contrasted with the number of real mutation occurred:
  #In the example we have:
  seq.lineage.1 <- substitutions.on.sequence(seq.ancestral,n.subst.lineage.1,Q,Tnt)
  seq.lineage.2 <- substitutions.on.sequence(seq.ancestral,n.subst.lineage.2,Q,Tnt)

  #The ancestral and new sequences:
  seq.ancestral
  seq.lineage.1
  seq.lineage.2

  #the number of observed and real differences differences:
  Diff <- matrix(0,nrow=3,ncol=2,dimnames=list(c("1.vs.2","anc.vs.1","anc.vs.2"),c("OBS","REAL")))
  Diff[1,1] <- sum(seq.lineage.1 != seq.lineage.2)
  Diff[2,1] <- sum(seq.ancestral != seq.lineage.1)
  Diff[3,1] <- sum(seq.ancestral != seq.lineage.2)
  Diff[1,2] <- n.subst.lineage.1 + n.subst.lineage.2
  Diff[2,2] <- n.subst.lineage.1
  Diff[3,2] <- n.subst.lineage.2
  Diff


  #################################################################################
  # 2.2.2 How can we infer the time of divergence using the observed differences? #
  #################################################################################
  #In order to have a correct estimate of the divergence time, we need to correct by the accumulation of substitutions in the same position
  #Let's see the effect across divergence time: '
  
  subst.rate <- 1e-6
  par(mfrow=c(1,1))
  G <- seq(from=1e3,to=1e6,by=(1e6-1e3)/29) #we subset 30 different divergence times
  y1 <- seq(from=0,to=2*subst.rate*G[30],by=2*subst.rate*G[30]/29)
  plot(x=2*G,y=y1,ylim=c(0,2),type="l",col="red",xlab="Time",ylab="Divergence")
  
  y2 <- c(3/4-3/4*exp(-4/3*subst.rate*2*G)) #theoretical curve
  lines(x=2*G,y=y2,ylim=c(0,2),type="l",col="black")

  #Simulate a new sequence of length 1000
  L <- 1e3
  seq.ancestral <- sample(x=Tnt,size=L,replace=TRUE,prob=c(0.25,0.25,0.25,0.25))
  #Jukes and Cantor rate matrix
  Q.JC <- matrix(1/3,ncol=4,nrow=4,dimnames=list(c("A","C","G","T"),c("A","C","G","T"))) 
  for(x in 1:4) {Q.JC[x,x] <- 0; Q.JC[x,x] <- -sum(Q.JC[x,])}

  #calculate divergence for different times (contained in the G vector)
  D.obs  <- array(0,30)
  D.real <- array(0,30)
  i <- 1
  for(g in G) {
    n.subst.lineage.1 <- rpois(n=1,lambda=subst.rate * L * g)
    n.subst.lineage.2 <- rpois(n=1,lambda=subst.rate * L * g)
    seq.lineage.1 <- substitutions.on.sequence(seq.ancestral,n.subst.lineage.1,Q.JC,Tnt)
    seq.lineage.2 <- substitutions.on.sequence(seq.ancestral,n.subst.lineage.2,Q.JC,Tnt)
    D.obs[i]  <-  sum(seq.lineage.1 != seq.lineage.2)/length(seq.ancestral)
    D.real[i] <-  (n.subst.lineage.1 + n.subst.lineage.2)/length(seq.ancestral)
    i <- i + 1
  }
  lines(x=2*G,y=D.real,type="p",col="red")
  lines(x=2*G,y=D.obs ,type="p",col="black")
  legend("topleft",legend=c("Real","Observed"),text.col=c("red","black"))

  ####################################################################################
  # 2.3. Including more realistic parameters
  # 2.3.1. The HKY model: adding differences in Ts/Tv and in the freqs of nucleotides
  ####################################################################################

  #A more complex model is the differentiation of transition and transversion substitutions ..
  #and the frequency of mutations. HKY model (Hasegawa, Kishino and Yano)
  #In this case, the substitution rate matrix is:
  #for example:
  p.A <- 0.55 #here fixed but they are usually parameters that have to be estimated
  p.C <- 0.06
  p.G <- 0.36
  p.T <- 1- (p.A + p.C + p.G)
  p.S <- 0.75 #here fixed but they are usually parameters that have to be estimated
  p.V <- 1 - p.S
  Q.HKY <- c(0,p.C*p.V,p.G*p.S,p.T*p.V,p.A*p.V,0,p.G*p.V,p.T*p.S,p.A*p.S,p.C*p.V,0,p.T*p.V,p.A*p.V,p.C*p.S,p.G*p.V,0)
  Q.HKY <- matrix(Q.HKY,ncol=4,nrow=4,byrow=TRUE,dimnames=list(c("A","C","G","T"),c("A","C","G","T")))
  for(x in 1:4) {Q.HKY[x,x] <- 0; Q.HKY[x,x] <- -sum(Q.HKY[x,])}
  Q.HKY #Substitution rate matrix for this specific parameters
    
  #######################################################################################################################################
  # Calculate the same plots shown the previous section but using the evolutionary model given by the HKY transition matrix. 
   #  Compare the results obtained with the theoretical values of the JC model.
 #######################################################################################################################################
  #ANSWER:
  subst.rate <- 1e-6
  par(mfrow=c(1,1))
  G <- seq(from=1e3,to=1e6,by=(1e6-1e3)/29) #we subset 30 different divergence times
  y1 <- seq(from=0,to=2*subst.rate*G[30],by=2*subst.rate*G[30]/29)
  plot(x=2*G,y=y1,ylim=c(0,2),type="l",col="red",xlab="Time",ylab="Divergence")
  
  y2 <- c(3/4-3/4*exp(-4/3*subst.rate*2*G)) #theoretical JC curve
  lines(x=2*G,y=y2,ylim=c(0,2),type="l",col="black")

  L <- 1e3
  seq.ancestral <- sample(x=Tnt,size=L,replace=TRUE,prob=c(p.A,p.C,p.G,p.T))

  D.obs  <- array(0,30)
  D.real <- array(0,30)
  i <- 1
  for(g in G) {
    n.subst.lineage.1 <- rpois(n=1,lambda=subst.rate * L * g)
    n.subst.lineage.2 <- rpois(n=1,lambda=subst.rate * L * g)
    seq.lineage.1 <- substitutions.on.sequence(seq.ancestral,n.subst.lineage.1,Q.HKY,Tnt)
    seq.lineage.2 <- substitutions.on.sequence(seq.ancestral,n.subst.lineage.2,Q.HKY,Tnt)
    D.obs[i]  <-  sum(seq.lineage.1 != seq.lineage.2)/length(seq.ancestral)
    D.real[i] <-  (n.subst.lineage.1 + n.subst.lineage.2)/length(seq.ancestral)
    i <- i + 1
  }
  lines(x=2*G,y=D.real,type="p",col="red")
  lines(x=2*G,y=D.obs ,type="p",col="black") #Observe that now the JC69 model does not fit!!
  #######################################################################################################################################


  ####################################################################################
  # 2.3.2. Including Heterogeneity across sequences
  ####################################################################################
  
  #Function: The number of mutations accumulate over the sequence with invariant positions:
  substitutions.on.sequence.plusI <- function(seq,I,substitutions,Q,Tnt) {
    #seq is a vector of symbols (eg., A, T, A, G, C ...etc.)
    #I is the ratio of invariant positions
    #substitutions is the total number of substitutions
    #Q is the substitution rate matrix (C x C)
    #Tnt is a vector of C values containing the different symbols in the seq, sorted as in Q
    seq1 <- seq
    len <- floor(length(seq)*(1-I))
    position.substitution <- sample(c(1:len),size=substitutions,replace=TRUE) #where the substitutions fall on seq
    number.symbols <- c(1:length(Tnt))
  
    for(ps in position.substitution) {
      nt <- which(Tnt==seq1[ps]) #which variant is in seq[ps]
      symbol <- number.symbols[-nt] #count the possible variants (not the original)
      seq1[ps] <- sample(Tnt[-nt],1,prob=Q[nt,-nt]) #assign the new nucleotide
    }   
    seq1 #the output is the new vector of symbols after including substitutions
  }
  
  ####################################################################################
  # 2.3.2... The Gamma distribution
  ####################################################################################
  
  #Finally, the models can also consider different ratios of substitutions across the length of the sequence. 
  #Ignoring variable evolutionary rates will underestimate the sequence distance.
  #This is summarized with one parameter using a standardized GAMMA distribution:
  #if shape=scale, the distribution has always mean 1 and the variance changes.

  par(mfrow=c(1,3))
  par.gamma <- c(100,1,0.01)
  for(i in 1:3) {
    alpha <- par.gamma[i] #mean=alpha/beta, var=alpha/beta^2
    beta <- alpha
    d <- rgamma(n=10000,shape=alpha,rate=beta)
    text.plot <- sprintf("Gamma distibution\nmean=%.2f, var=%.3f",mean(d),var(d))
    hist(d,main=text.plot)
  }
  #plot 1 is almost fixed, plot 3 is a rate with many (almost) invariant positions and few hotspots

  ####################################################################################
  # 2.3.2... The function that calculate substitutions considering heterogeneity
  ####################################################################################

  substitutions.on.sequence.plusI.Gamma <- function(seq,I,Gshape=0,substitutions,Q,Tnt) {
    #seq is a vector of symbols (eg., A, T, A, G, C ...etc.)
    #I is the ratio of invariant positions
    #Gshape is the parameter shape=scale in a gamma distribution
    #substitutions is the total number of substitutions
    #Q is the substitution rate matrix (C x C)
    #Tnt is a vector of C values containing the different symbols in the seq, sorted as in Q
    seq1 <- seq
    len <- floor(length(seq)*(1-I))
    
    #including gamma:
    if(Gshape > 0) {
      position.substitution <- array(,substitutions)
      pg <- rgamma(n=len,shape=Gshape,rate=Gshape) #each position has a value
      wt <- pg/sum(pg) #each position has a weight according to the gamma distribution
      position.substitution <- sample(c(1:len),size=substitutions,replace=TRUE,prob=wt) #sample each position according its weight
    } else {
      position.substitution <- sample(c(1:len),size=substitutions,replace=TRUE) #where the substitutions fall on seq
    }
    number.symbols <- c(1:length(Tnt))
  
    for(ps in position.substitution) {
      nt <- which(Tnt==seq1[ps]) #which variant is in seq[ps]
      symbol <- number.symbols[-nt] #count the possible variants (not the original)
      seq1[ps] <- sample(Tnt[-nt],1,prob=Q[nt,-nt]) #assign the new nucleotide
    }   
    seq1 #the output is the new vector of symbols after including substitutions
  }

  #########################################################################################
  # 2.3.3. Differences in the divergence estimation when ewe fail to use the correct model
  #########################################################################################

  #Example Plot: JC + gamma + Invariant positions
  par(mfrow=c(1,1))
  L <- 1e3
  seq.ancestral <- sample(x=Tnt,size=L,replace=TRUE,prob=c(0.25,0.25,0.25,0.25))
  subst.rate <- 1e-6
  mu <- 1e-6 
  ratio.invariant <- 0.3
  Gamma.shape <- c(1)

  G <- seq(from=1e3,to=1e6,by=(1e6-1e3)/29) #we subset 30 different divergence times
  y1 <- seq(from=0,to=2*subst.rate*G[30],by=2*subst.rate*G[30]/29)
  plot(x=2*G,y=y1,ylim=c(0,2),type="l",col="black",xlab="Time",ylab="Divergence")

  y2 <- c(3/4-3/4*exp(-4/3*subst.rate*2*G)) #theoretical curve JC
  lines(x=2*G,y=y2,ylim=c(0,2),type="l",col="cyan")
  y3 <- c(3/4*(1-ratio.invariant)-3/4*(1-ratio.invariant)*exp(-4/(3*(1-ratio.invariant))*subst.rate*2*G)) #theoretical curve + invariant
  lines(x=2*G,y=y3,ylim=c(0,2),type="l",col="blue")
  y4 <- c(3/4*(1-ratio.invariant)*(1-(4/(3*(1-ratio.invariant))*subst.rate*2*G/Gamma.shape+1)^-Gamma.shape)) #theoretical curve + invariant + gamma
  lines(x=2*G,y=y4,ylim=c(0,2),type="l",col="red")

  D.obs.JC.gamma  <- array(0,30)
  D.real <- array(0,30)
  i <- 1
  for(g in G) {
    n.subst.lineage.1 <- rpois(n=1,lambda=subst.rate * L* (1-ratio.invariant) * g) #gamma is not included because the mean = subs.rate
    n.subst.lineage.2 <- rpois(n=1,lambda=subst.rate * L* (1-ratio.invariant) * g)
    D.real[i] <-  (n.subst.lineage.1 + n.subst.lineage.2)/(length(seq.ancestral)*(1-ratio.invariant))
    
    seq.lineage.1.JC <- substitutions.on.sequence.plusI.Gamma(seq.ancestral,I=ratio.invariant,Gamma.shape,n.subst.lineage.1,Q.JC,Tnt)
    seq.lineage.2.JC <- substitutions.on.sequence.plusI.Gamma(seq.ancestral,I=ratio.invariant,Gamma.shape,n.subst.lineage.2,Q.JC,Tnt)
    D.obs.JC.gamma[i]  <-  sum(seq.lineage.1.JC != seq.lineage.2.JC)/(length(seq.ancestral)*(1-0))    
        
    i <- i + 1
  }
  lines(x=2*G,y=D.obs.JC.gamma,type="p",col="red")
  lines(x=2*G,y=D.real,type="p",col="black")


  #########################################################################################################
  # 2.4. MODEL SELECTION                                                                                  #
  # 2.4.1. The Likelihood of the data for a given model                                                   #
  #########################################################################################################

  #The last point of this section is to learn how to distinguish the best evolutionary model for our data
  #Observed data:
  L <- 1e3
  x <- 300 #D.obs = 300/1e3 = 0.3
  nS <- 125 #number of transitions
  nV <- x-nS #number of transversions
  
  par(mfrow=c(1,3))

  ################
  # The JC model #
  ################
  #In case JC, the probability that a position has a different nucleotide between 2 seqs is:
  #D.obs <- 3/4 - 3/4*exp(-4/3*D.real)
  #There are two states, we use a binomial to calculate the probability:
  #f(x|D.real) <- Const * D.obs^x * (1-D.obs)^(L-x)
  #in order to compare all models, the equation is redefined to consider all 4x4 cell of the matrix using a multinomial
  
  nit <- 1e5
  D.real <- runif(nit) #Here we simulate 1e5 values of divergence given the observed values
  logLik <- x * log(1/16 - 1/16*exp(-4/3*D.real)) + (L-x) * log(1/16 + 3/16*exp(-4/3*D.real))

  p.ML <- which(logLik == max(logLik))
  ML.1 <- max(logLik)
  MLE.1 <- D.real[p.ML]
  ML.1
  MLE.1

  #Also, the confident interval can be obtained by lowering the MLE by X-square(1dgf,5%)/2 = 1.92: (max(logLik) - 1.92)
  Ci <- logLik
  Ci[which(Ci > max(logLik) - 1.92)] = min(logLik)
  
  cil <- 1
  cir <- 1
  for(i in 1:nit) {
    if(D.real[i] < MLE.1 && Ci[i] > Ci[cil]) cil <- i  
    if(D.real[i] > MLE.1 && Ci[i] > Ci[cir]) cir <- i  
  }

  D.real.int.chi2.1 <- c(0,0) 
  D.real.int.chi2.1[1] <- D.real[cil]
  D.real.int.chi2.1[2] <- D.real[cir]

  plot(D.real,logLik,cex = .1,main="Model JC69")
  abline(v=D.real.int.chi2.1[1],col="red")
  abline(v=D.real.int.chi2.1[2],col="red")
  text(D.real.int.chi2.1[1],min(logLik),sprintf("Dl=%.3f",D.real.int.chi2.1[1]),adj = c(1,0))
  text(D.real.int.chi2.1[2],min(logLik),sprintf("Dr=%.3f",D.real.int.chi2.1[2]),adj = c(0,0))
  abline(h=max(logLik) - 1.92,col="red")
  text(min(D.real),max(logLik),sprintf("Lk=%.3f",max(logLik)),adj = c(0,0))

  #################
  # The K2P model #
  #################

  #We try our data with a model Kimura1980: Parameters are the divergence and the ratio transition(S)/transversion(V)
  #In case using the model K80, the probability that a position has a different nucleotide between 2 seqs is:
  #D.obs <- 1/2*log(1-2*S-V) - 1/4*log(1-2*V)
  #The likelihood function is:

  nit <- 1e5
  D.real <- runif(nit)
  k <- runif(nit,-1,1)
  k <- 10^k
  p0 <- 1/4+1/4*exp(-4*D.real/(k+2))+1/2*exp(-2*D.real*(k+1)/(k+2))
  p1 <- 1/4+1/4*exp(-4*D.real/(k+2))-1/2*exp(-2*D.real*(k+1)/(k+2))
  p2 <- 1/4-1/4*exp(-4*D.real/(k+2))
  logLik <- (L-nS-nV) * log(p0/4) + nS * log(p1/4) + nV * log(p2/4)

  p.ML <- which(logLik == max(logLik))
  ML.2 <- max(logLik)
  MLE.21 <- D.real[p.ML]
  MLE.22 <- k[p.ML]
  ML.2
  MLE.21
  MLE.22

  D.real.int.chi2.2 <- c(0,0,0,0) 
  #The confident intervals can be obtained by lowering the MLE by X-square(1dgf,5%)/2 = 1.92: (max(logLik) - 1.92)
  Ci <- logLik
  Ci[which(Ci > max(logLik) - 1.92)] = min(logLik)

  ############ first parameter:
  cil <- 1
  cir <- 1
  for(i in 1:nit) {
    if(D.real[i] < MLE.21 && Ci[i] > Ci[cil]) cil <- i  
    if(D.real[i] > MLE.21 && Ci[i] > Ci[cir]) cir <- i  
  }

  D.real.int.chi2.2[1] <- D.real[cil]
  D.real.int.chi2.2[2] <- D.real[cir]

  plot(D.real,logLik,cex = .1,main="Model K80")
  abline(v=D.real.int.chi2.2[1],col="red")
  abline(v=D.real.int.chi2.2[2],col="red")
  text(D.real.int.chi2.2[1],min(logLik),sprintf("Dl=%.3f",D.real.int.chi2.2[1]),adj = c(1,0))
  text(D.real.int.chi2.2[2],min(logLik),sprintf("Dr=%.3f",D.real.int.chi2.2[2]),adj = c(0,0))
  abline(h=max(logLik) - 1.92,col="red")
  text(min(D.real),max(logLik),sprintf("Lk=%.3f",max(logLik)),adj = c(0,0))

  ########### second parameter:
  cil <- 1
  cir <- 1
  for(i in 1:nit) {
    if(k[i] < MLE.22 && Ci[i] > Ci[cil]) cil <- i  
    if(k[i] > MLE.22 && Ci[i] > Ci[cir]) cir <- i  
  }
  
  D.real.int.chi2.2[3] <- k[cil]
  D.real.int.chi2.2[4] <- k[cir]

  plot(k,logLik,cex = .1,main="Model K80")
  abline(v=D.real.int.chi2.2[3],col="red")
  abline(v=D.real.int.chi2.2[4],col="red")
  text(D.real.int.chi2.2[3],min(logLik),sprintf("kl=%.3f",D.real.int.chi2.2[3]),adj = c(1,0))
  text(D.real.int.chi2.2[4],min(logLik),sprintf("kr=%.3f",D.real.int.chi2.2[4]),adj = c(0,0))
  abline(h=max(logLik) - 1.92,col="red")
  text(min(D.real),max(logLik),sprintf("Lk=%.3f",max(logLik)),adj = c(0,0))

  #########################################################################################################
  # 2.4.1. The Likelihood Ratio Test (LRT) to estimate the best model                                     #
  #########################################################################################################

  #Finally, we can compare the likelihoods of the two models using a likelihood ratio test (LRT)
  #The significance can be obtained from a Chi-square with the differece between models as degree of freedom 
  
  ML.1
  ML.2
  Xi2 <- 2*(ML.2-ML.1)
  1-pchisq(Xi2,1) #Probability that the null model (JC) is accepted 
 