######################################################################
######################## 3. PHYLOGENY ############################
######################################################################

#First include a function from the previous section that will use to simulate the number of substitutions in a branch

#Function: The number of mutations accumulate over the sequence with invariant positions :
#and also a variable substitution rate given by a gamma distribution:
substitutions.on.sequence.plusI.Gamma <- function(seq,I,Gshape=0,substitutions,Q,Tnt) {
    #seq is a vector of symbols (eg., A, T, A, G, C ...etc.)
    #I is the ratio of invariant positions
    #Gshape is the parameter shape=scale in a gamma distribution
    #substitutions is the total number of substitutions
    #Q is the substitution rate matrix (C x C)
    #Tnt is a vector of C values containing the different symbols in the seq, sorted as in Q
    seq1 <- seq
    len <- floor(length(seq)*(1-I))
    
    #including gamma:
    if(Gshape > 0) {
      position.substitution <- array(,substitutions)
      pg <- rgamma(n=len,shape=Gshape,rate=Gshape) #each position has a value
      wt <- pg/sum(pg) #each position has a weight according to the gamma distribution
      position.substitution <- sample(c(1:len),size=substitutions,replace=TRUE,prob=wt) #sample each position according its weight
    } else {
      position.substitution <- sample(c(1:len),size=substitutions,replace=TRUE) #where the substitutions fall on seq
    }
    number.symbols <- c(1:length(Tnt))
  
    for(ps in position.substitution) {
      nt <- which(Tnt==seq1[ps]) #which variant is in seq[ps]
      symbol <- number.symbols[-nt] #count the possible variants (not the original)
      seq1[ps] <- sample(Tnt[-nt],1,prob=Q[nt,-nt]) #assign the new nucleotide
    }   
    seq1 #the output is the new vector of symbols after including substitutions
}

###################################################################
# 3.4. Create a history and simulate sequences 
###################################################################

#We want to study the following phylogeny:
#Newick format
tree.phyl <- "(((1,2),(3,(4,5))),6);"
tree.phyl.time <- "(((1:8e5,2:8e5):7.2e6,(3:7e6,(4:6e6,5:6e7):1e6):1e6):1.2e7,6:2e7);" #in generations

par(mfrow=c(2,1))
library(ape) #plot the real species tree
tree.phyl.time.T <- "(((1:8e-1,2:8e-1):7.2e0,(3:7e0,(4:6e0,5:6e0):1e0):1e0):1.2e1,6:2e1);" #in 1e6_generations
cat(tree.phyl.time.T, file = "tree.phyl.time.T.phy", sep = "\n")
tree.phyl.time.T <- read.tree("tree.phyl.time.T.phy") # load tree
unlink("tree.phyl.time.T.phy") # delete the file 
plot(tree.phyl.time.T)

tree.phyl.time.D <- "(((1:0.024,2:0.016):0.144,(3:0.210,(4:0.06,5:0.06):0.02):0.01):0.12,6:0.2);" #in divergence/nt
cat(tree.phyl.time.D, file = "tree.phyl.time.D.phy", sep = "\n")
tree.phyl.time.D <- read.tree("tree.phyl.time.D.phy") # load tree
unlink("tree.phyl.time.D.phy") # delete the file 
plot(tree.phyl.time.D)

#Tree example: include the nodes and relative times BUT ALSO mu and Ne: That is, no real molecular clock and the possibility of incomplete lineage sorting
n.lineages <- 6
tree.data <- matrix(c(
  1,    1,   7,    8e5,   3e-8,   1e4,
  2,    2,   7,    8e5,   2e-8,   1e3,
  3,    3,   9,    7e6,   3e-8,   1e4,
  4,    4,   8,    6e6,   1e-8,   1e4,
  5,    5,   8,    6e6,   1e-8,   1e5,
  6,    6,  11,    2e7,   1e-8,   1e4,
  1,    2,  10,  7.2e6,   2e-8,   5e3,
  4,    5,   9,    1e6,   2e-8,   1e5,
  3,    8,  10,    1e6,   1e-8,   1e5,
  7,    9,  11,  1.2e7,   1e-8,   1e5,
  6,   10,   0,      0,   1e-8,   1e4  #outgroup
  ),nrow=2*n.lineages-1,ncol=6,dimnames=list(c(sprintf("Node%02d",c(1:(2*n.lineages-1)))),c("node1","node2","go.to.node","Time.sp","mu","Ne")),byrow=TRUE) 
tree.data <- data.frame(cbind(tree.data[,1:6]))

tree.data

###################################################################
# 3.4... Calculate the divergence times 
###################################################################

#counting the time period of coalescence of nodes (that is, from present to past, after species join, count when the sequences coalesce)
#COALESCENCE of lineages in a population, prob=1/Ne in haploid population
#the expected time of coalescence follows an exponential distribution (the expected time that something random occurs)
tree.data <- data.frame(cbind(tree.data[,c(1:6)],Time.seq=tree.data$Time.sp))
tree.data

for(lg in (n.lineages+1):(2*n.lineages-1)) {
  t1 <- tree.data$Time.seq[tree.data$node1[lg]] #time speciation first ind.
  t2 <- tree.data$Time.seq[tree.data$node2[lg]] #time speciation second ind.
  time.coal <- rexp(n=1,rate=1/tree.data$Ne[lg]) #TIME COALESCENCE both sequences in the joined population
  tree.data$Time.seq[tree.data$node1[lg]] <- t1 + time.coal   #adding time of coalescence to the first and second nodes
  tree.data$Time.seq[tree.data$node2[lg]] <- t2 + time.coal   #adding time of coalescence to the first and second nodes
  #tree.data$Time.seq[lg] <- ifelse(tree.data$Time.seq[lg] - time.coal > 0,tree.data$Time.seq[lg] - time.coal,0) #eliminate the time of coalescence to the next node
}
tree.data

###################################################################
# 3.4... Calculate the number of substitutions 
###################################################################
#counting the number of substitutions occurred in each branch
tree.data <- data.frame(cbind(tree.data[,c(1:7)],mutations=rep(NA,2*n.lineages-1)))  
L <- 1e3 #The length of the sequence to study is L
tree.data$mutations <- rpois(n=2*n.lineages-1,lambda=tree.data$mu * tree.data$Time.seq * L)
tree.data

###################################################################
# 3.4... Construct the sequences 
###################################################################
## CONSTRUCT SEQUENCES ##
#We want to know what are the sequences we have from this tree, assuming a mutational model, for example JC69
Tnt <- c("A","C","G","T")
seq1 <- sample(x=Tnt,size=L,replace=TRUE,prob=c(0.25,0.25,0.25,0.25)) #Starting form a sequence, we include mutations
seq <- matrix(rep(seq1,2*n.lineages-1),ncol=L,byrow=TRUE)
#Start to include mutation to each node according the order of nodes, from past to present:
Q.JC <- matrix(1/3,ncol=4,nrow=4,dimnames=list(Tnt,Tnt)) #Jukes and Cantor rate matrix
for(x in 1:4) {Q.JC[x,x] <- 0; Q.JC[x,x] <- -sum(Q.JC[x,])}
for(node in (2*n.lineages-2):1) {
  seq[node,] <- substitutions.on.sequence.plusI.Gamma(seq=seq[node,],I=0,Gshape=0,substitutions=tree.data$mutations[node],Q=Q.JC,Tnt=Tnt)
  seq[tree.data$node1[node],] <- seq[node,]
  seq[tree.data$node2[node],] <- seq[node,]
}
seq[,1:50]

###################################################################
# 3.4... Function to calculate the divergence under the JC model 
###################################################################
#function calculating diff and divergence using JC69:
diffs.plus.jc69.div.matrix <- function(seq.matrix) {
  n.lineages <- (length(seq.matrix[,1])+1)/2
  L <- length(seq.matrix[1,])
  
  D <- matrix(0,nrow=n.lineages,ncol=n.lineages)
  #Calculate the matrix of differences between seqs
  for(i in 1:(n.lineages-1)) {
    for(j in (i+1):n.lineages) {
      D[i,j] <- sum(seq.matrix[i,] != seq.matrix[j,])
    }
  }
  #Calculate the divergence matrix assuming the model JC69:
  for(i in 1:(n.lineages-1)) {
    for(j in (i+1):n.lineages) {
      D[j,i] <- -3/4*log(1-4/3*D[i,j]/L) #Jukes and Cantor correction
    }
  }
  D
}

D <- diffs.plus.jc69.div.matrix(seq)
D
for(i in 1:(length(D[1,])-1)) for(j in (i+1):length(D[,1])) D[i,j] <- D[j,i]
D


#########################################################################
# 3.5. Methods for Reconstructing Phylogenetic trees
# 3.5.3. Practical Exercises. the UPMA method
#########################################################################

#From Distance matrix:
#UPGMA. Strict molecular clock

#Steps: from distance matrix, find the shortest distance, recalculate the matrix and average the closest, repeat until having 1x1 matrix
#how to keep data and represent data: matrix with rows indicating the nodes and the distance between them, 
#plot, assign position to each lineage, average position values with internal nodes, vertical lines is the distance, horizontal the unions
locate.min.lower.matrix <- function(D.tmp) {
  if(is.null(rownames(D.tmp))) rownames(D.tmp) <- c(1:length(D.tmp[,1]))
  if(is.null(colnames(D.tmp))) colnames(D.tmp) <- c(1:length(D.tmp[1,]))
  l <- length(D.tmp[1,])
  if(l<2) stop("too low dimensions")
  min.value <- D.tmp[2,1]
  row.value <- 2
  col.value <- 1
  for(i in 1:(l-1)) { #cols
    for(j in (i+1):l) { #rows
      if(min.value > D.tmp[j,i]) {
        min.value <- D.tmp[j,i]
        row.value <- j
        col.value <- i
      }
    }
  }
  i <- as.numeric(rownames(D.tmp)[row.value])
  j <- as.numeric(colnames(D.tmp)[col.value])
  c(i,j,min.value)
}

do.tree.matrix.upgma <- function(D) {
  n.lineages <- length(D[1,])
  tree.matrix <- matrix(0,nrow=2*n.lineages-1,ncol=4,dimnames=list(c(1:(2*n.lineages-1)),c("node1","node2","go.to.node","Div"))) #matrix all relationships
  tree.matrix[1:n.lineages,1:2] <- c(1:n.lineages) #init
  cluster <- matrix(0,nrow=n.lineages,ncol=n.lineages+1,dimnames=list(sprintf("line%d",c(1:n.lineages)),sprintf("cluster%d",c(0:(n.lineages)))))  #clustering matrix
  cluster[1:n.lineages,1:2] <- c(1:n.lineages) #init
  
  D.tmp <- D
  rownames(D.tmp) <- c(1:length(D.tmp[,1]))
  colnames(D.tmp) <- c(1:length(D.tmp[1,]))

  l <- n.lineages
  n <- 2
  while(length(D.tmp[1,]) > 1) {
    Dmin <- locate.min.lower.matrix(D.tmp) #find the minimum value and their cluster coordinates
    join.clust <- c(which(cluster[,n] == Dmin[1]),which(cluster[,n] == Dmin[2])) #locate all lineages in the cluster
    node.number <- n+n.lineages-1
    cluster[join.clust,n] <- node.number #assign a new node to the new cluster
    cluster[,n+1] <- cluster[,n] #copy the column for next step
     
    #do the new matrix averaging the clustering lineages using the original D matrix
    D.tmp <- matrix(0,l-1,l-1)
    nodes <- sort(unique(cluster[,n])) #find the nodes that are currently present
    colnames(D.tmp) <- t(nodes)
    rownames(D.tmp) <- t(nodes)
    ii <- 1
    for(i in as.numeric(t(nodes))) {
      jj <- 1
      for(j in as.numeric(t(nodes))) {
        D.tmp[jj,ii] <- mean(D[which(cluster[,n]==j),which(cluster[,n]==i)])
        jj <- jj + 1
      }
      ii <- ii + 1
    }
    #D.tmp  
    #calculating the new tree matrix   
    tree.matrix[node.number,1:2] <- Dmin[1:2]
    nodes.prec <- sort(unique(cluster[which(cluster[,n]==node.number),n-1]))
    tree.matrix[nodes.prec ,  3] <- node.number
    tree.matrix[nodes.prec,   4] <- Dmin[3]/2
    
    n <- n + 1
    l <- l - 1
  }
  #relative divergence for each node
  for(i in (2*n.lineages-2):(n.lineages+1)) {
    tree.matrix[i,4] <- tree.matrix[i,4] - tree.matrix[max(tree.matrix[i,1:2]),4]
  }
  #output a tree.data variable with the nodes and the distances and the clustering matrix
  list(tree.data=as.data.frame(tree.matrix),cluster=cluster[,-length(cluster[1,])])
}

UPGMA.tree <- do.tree.matrix.upgma(D)
UPGMA.tree

#NJ: no molecular clock. It is the best tree if distance data matrix is additive (d_ij = d_ix + d_xj)
#Don't do  here
#ML, Bayesian: MCMC
#Don't do  here

#Ploting using the libraries "ape" and "phangorn":
library(ape)
library(phangorn)

par(mfrow=c(1,1))
UPGMA.D <- upgma(D)
str(UPGMA.D) #arguments in output
UPGMA.D$edge #relationship among nodes
plot(UPGMA.D)

Dd <- as.dist(lower.tri(D) * D)
NJ.D <- nj(Dd) #relationship among nodes
str(NJ.D)  #arguments in output
NJ.D$edge
plot(NJ.D,"u")

#########################################################################
# 3.6. Support of the phylogenetic tree inferred: Bootstrap
#########################################################################

#Bootstrap
#The simplest bootraping method consist in picking columns from the sequence data with replacement and reconstruct a new dataset
bootstrap.seq.matrix <- function(seq.matrix) {
  L <- length(seq.matrix[1,])
  post <- sample(x=c(1:L),size=L,replace=TRUE)
  seq.matrix <- seq.matrix[,post]
  seq.matrix
}

#We want to use bootstrap to calculate the confidence of the observed tree for my locus:
#recursive function to look for external nodes
look.for.external.nodes <- function(tree.observed,i) {
  external.nodes <- 0
  if(tree.observed$node2[tree.observed$node1[i]] == tree.observed$node1[i]) {
    external.nodes <- c(external.nodes,tree.observed$node1[i])
  } else {
    external.nodes <- c(external.nodes,look.for.external.nodes(tree.observed,tree.observed$node1[i]))
  }
  if(tree.observed$node2[tree.observed$node2[i]] == tree.observed$node1[i]) {
    external.nodes <- c(external.nodes,tree.observed$node2[i])
  } else {
    external.nodes <- c(external.nodes,look.for.external.nodes(tree.observed,tree.observed$node2[i]))
  }
  external.nodes[-1]
}

#function to compare internal tree nodes
compare.tree.nodes <- function(tree.observed,tree.check) {
  n.lineages <- length((tree.observed[,1]+1)/2)
  if(is.null(tree.observed$bootstrap)) tree.observed <- data.frame(cbind(tree.observed,bootstrap=0)) ##add a new field
  for(i in (2*n.lineages-2):(n.lineages+1)) { #compare all internal nodes
    internal.nodes.obs <- sort(unique(look.for.external.nodes(tree.observed,i)))
    internal.nodes.chk <- sort(unique(look.for.external.nodes(tree.check,i)))
    if(length(internal.nodes.obs) == length(internal.nodes.chk)) {
      if(sum(internal.nodes.obs != internal.nodes.chk) == 0) {
        tree.observed$bootstrap[i] <- tree.observed$bootstrap[i] + 1
      }
    }
  } 
  tree.observed
}

iter <- 1e3
for(i in 1:iter) {
  seq.boot <- bootstrap.seq.matrix(seq)
  #calculate the distance matrix
  D.boot <- diffs.plus.jc69.div.matrix(seq.boot)
  #do symmetric matrix
  for(i in 1:(length(D.boot[1,])-1)) for(j in (i+1):length(D.boot[,1])) D.boot[i,j] <- D.boot[j,i]
  #calculate the number of equal clustered internal nodes in relation to the observed data
  UPGMA.boot <- do.tree.matrix.upgma(D.boot)
  #UPGMA.boot
  tree.data <- compare.tree.nodes(tree.observed=tree.data,tree.check=UPGMA.boot$tree.data)
}
tree.data$bootstrap <- tree.data$bootstrap/iter
tree.data #look at bootstrap support

#########################################################################
# 3.7. Phylogenomics: a way to obtain the Species Tree
#########################################################################

#Phylogenomics: uncorrelated loci histories can solve the problem of Incomplete Lineage sorting by estimating internal Ne and thus the species tree
#First download phybase at https://faculty.franklin.uga.edu/lliu/content/phybase?
#then install dependencies (ape and Matrix) using 
install.packages("Matrix")
#Finally install phybase
install.packages("./phybase_1.4.tar",repos=NULL,type="source")
library(phybase) #use star.sptree function: STAR algorithm

#Find the species tree by obtaining 100 loci from tree.data
#Calculate a NJ tree for each one using "ape" and "phybase" libraries
#Then use the STAR algorithm by mean of star.sptree function in phybase to calculate the consensus species tree.

tree.data <- data.frame(cbind(tree.data[,1:6]))
tree.data <- data.frame(cbind(tree.data,Time.seq=0,mutations=0))
mu.vector <- tree.data$mu

n.loci <- 100
#Dd.Dl <- array(,dim=c(6,6,n.loci))
NJ.Dl <- array(,dim=c(n.loci))
locus <- 1
while(locus <= n.loci) {
  tree.data$Time.seq <- tree.data$Time.sp
  #from tree.data
  for(lg in (n.lineages+1):(2*n.lineages-1)) {
    t1 <- tree.data$Time.seq[tree.data$node1[lg]] #time speciation first ind.
    t2 <- tree.data$Time.seq[tree.data$node2[lg]] #time speciation second ind.
    time.coal <- rexp(n=1,rate=1/tree.data$Ne[lg]) #TIME COALESCENCE both sequences in the joined population
    tree.data$Time.seq[tree.data$node1[lg]] <- t1 + time.coal   #adding time of coalescence to the first and second nodes
    tree.data$Time.seq[tree.data$node2[lg]] <- t2 + time.coal   #adding time of coalescence to the first and second nodes
    #tree.data$Time.seq[lg] <- ifelse(tree.data$Time.seq[lg] - time.coal > 0,tree.data$Time.seq[lg] - time.coal,0) #eliminate the time of coalescence to the next node
  }
  
  #L <- 1e3
  L <- floor(runif(1,400,2000)) #The length of the sequence to study is L, different in each locus
  tree.data$mu <- mu.vector*10^(runif(1,-0.5,0.5)) #changing the mutation rate in each locus
    
  tree.data$mutations <- rpois(n=2*n.lineages-1,lambda=tree.data$mu * tree.data$Time.seq * L)
  
  Tnt <- c("A","C","G","T")
  seq1 <- sample(x=Tnt,size=L,replace=TRUE,prob=c(0.25,0.25,0.25,0.25)) #Starting form a sequence, we include mutations
  seq <- matrix(rep(seq1,2*n.lineages-1),ncol=L,byrow=TRUE)
  
  #Start to include mutation to each node according the order of nodes, from past to present:
  Q.JC <- matrix(1/3,ncol=4,nrow=4,dimnames=list(Tnt,Tnt)) #Jukes and Cantor rate matrix
  for(x in 1:4) {Q.JC[x,x] <- 0; Q.JC[x,x] <- -sum(Q.JC[x,])}
  for(node in (2*n.lineages-2):1) {
    seq[node,] <- substitutions.on.sequence.plusI.Gamma(seq=seq[node,],I=0,Gshape=0,substitutions=tree.data$mutations[node],Q=Q.JC,Tnt=Tnt)
    seq[tree.data$node1[node],] <- seq[node,]
    seq[tree.data$node2[node],] <- seq[node,]
  }
  D <- diffs.plus.jc69.div.matrix(seq)
  
  if(sum(apply(D,c(1,2),is.nan)) == 0) {  
    for(i in 1:(length(D[1,])-1)) for(j in (i+1):length(D[,1])) D[i,j] <- D[j,i]
    Dd <- as.dist(lower.tri(D) * D)
  
  	#Dd.Dl[,,locus] <- as.matrix(Dd)
    NJ.Dl[locus] <- write.tree(root(phy=nj(Dd),outgroup=6),file="")
    locus <- locus + 1
  }
}
speciesname<-species.name(NJ.Dl[1])
taxaname<-speciesname
species.structure<-matrix(0,ncol=n.lineages,nrow=n.lineages)
diag(species.structure)<-1

#Finally, collect all trees and calculate the species tree
multilocus.tree <- star.sptree(NJ.Dl, speciesname, taxaname, species.structure,outgroup="6",method="nj") 
#Plot the real and estimated trees
par(mfrow=c(2,1))
plot(tree.phyl.time.T)
#plot(tree.phyl.time.D)
plottree(multilocus.tree)


#########################################################################
# 3.8. Final Exercise
#########################################################################

#######################################################################################################################################
comment <- c("
Download, in fasta format, the dataset for the complete mitochondrial genome of a single individual of the several available primate 
species (Eukaryota; Metazoa; Chordata; Craniata; Vertebrata; Euteleostomi; Mammalia; Eutheria; Euarchontoglires; Primates) and 
one outgroup (Sus scrofa). Here, use Homo sapiens, Pan troglodytes, Lemur catta and Papio hamadryas. Do a multiple alignment 
(for example using Tcoffe or MAFFT). Filter the poorly aligned positions and divergent regions (e.g. using Gblocks) and then estimate 
the best evolutionary model (for example using jModelTest). Do a ML phylogenetic reconstruction (e.g. using PhyML). 
Calculate bootstrap support for each node. Modify the plot using FigTree, iTol or another application and include the length of the 
branches and the bootstrap values.
Evaluate the tree (divergence values, evolutionary model, molecular clock and bootstrap support). 
Results of the exercises will be given in a PDF file containing the methods used, the results (a tree) and the discussion evaluating 
several of the analysis performed here.
")
#######################################################################################################################################
