######################################################################################
############ Basic concepts of Population Genetics useful for Phylogeny ##############
######################################################################################

###################################
############ Mutation: ############
###################################
#In a population, the number of mutations depend on the mutation rate and the number of individuals (and also the number of nucleotides per individual)
mu <- 1e-3 #mutations per generation per position
N <- 1e3 #individuals in the population
L <- 1e1 #positions per individual
expected.m <- mu * N * L
expected.m # is the average number of mutations in the population per generation in the region of size L
#mutation is a random process, therefore for each generation we will have a random value centered on expected.m
m <- rpois(n=1,lambda=mu*N*L)
m #will be for a specific case

######################################################################################
# EXERCISE 1: Simulate the number of mutations in 1000 generations
######################################################################################
#ANSWER:
mu <- 1e-3 #mutations per generation per position
N <- 1e3 #individuals in the population
L <- 1e1 #positions per individual
#the number of expected mutations in G generations would be:
G <- 1e3 #number of generations
expected.m.G <- mu * N * L * G
expected.m.G #the total number of mutations expected in G generations for the population
m.G <- rpois(n=1,lambda=mu*N*L*G)
m.G #will be for a specific case

######################################################################################

###################################
######## Drift to fixation ########
###################################
#Some of these mutations will be lost in the next generation and others will remain: the probability that a given mutation survive is 
#the probability that the individual carrying the mutation survive: 1/N (probability this individual contribute to the next N individuals)
#One mutation is fixed when the frequency of the mutation increase to 1 (all individuals have the mutation)
#One mutation is lost when the frequency drops to 0 (any individual has the mutation)

#Practical simulation case:
#Given a mutation in a position, look at the changes in the frequency of mutation per generation
#first a function to calculate the fixation time:
calculate.fixation.time.p <- function(initial.p,N,max.generations) {
  plotf <- array(0,dim=c(max.generations)) 
  p <- initial.p
  g <- 1
  plotf[g] <- p
  while(p > 0 && p < 1 && g < max.generations) {
    Pop <- rbinom(n=N,size=1,prob=p) #a binomial, equivalent to see what is the ancestor (0 or 1) for each individual in the new generation 
    p <- sum(Pop)/N
    g <- g + 1
    plotf[g] <- p
  }
  list(end.p=p,generations=g,plot.freqs=plotf)
}

#start simulation
par(mfrow=c(1,3))
it <- 1e4
max.g <- 1e4
plotf <- array(0,dim=c(max.g))
plot(x=c(1:max.g),rep(0,max.g),ylim=c(0,1),type="l")
dist.fix <- array(0,dim=c(it))
n.fix <- 0
dist.los <- array(0,dim=c(it))
n.los <- 0

for(i in 1:it) { 
  nm <- 1 #given a mutation, frequency in the initial population
  p <- nm/N
  
  fix.results <- calculate.fixation.time.p(initial.p=p,N=N,max.generations=max.g)
  plotf <- fix.results$plot.freqs
  g     <- fix.results$generations
  p     <- fix.results$end.p
  
  if(p == 1) {
    lines(x=c(1:g),y=plotf[1:g],type="l",col="black")
    dist.fix[n.fix+1] <- g
    n.fix <- n.fix + 1
  }
  if(p == 0) {
    lines(x=c(1:g),y=plotf[1:g],type="l",col="yellow")
    dist.los[n.los+1] <- g
    n.los <- n.los + 1
  }
}

hist(dist.fix[1:n.fix],breaks=30,main=sprintf("dist.fix: n=%d; mean=%.3f",n.fix,mean(dist.fix[1:n.fix])),freq=FALSE,xlim=c(0,max.g))
abline(v=mean(dist.fix[1:n.fix]),col="black")
n.fix
hist(dist.los[1:n.los],breaks=30,main=sprintf("dist.los: n=%d; mean=%.3f",n.los,mean(dist.los[1:n.los])),freq=FALSE,xlim=c(0,max.g))
abline(v=mean(dist.los[1:n.los]),col="yellow")
n.los

#proportion of fixed mutations
n.fix/(n.fix+n.los) #1/N under neutrality (total 1/N * it)
#number of generations to fix mutations
mean(dist.fix[1:n.fix]) #2*N under neutrality

######################################################################################
# EXERCISE 2: Change the population size (N) and the scale of the plots (xlim in the hist function) to see the effect. 
#Compare the fraction of fixed and the mean time to fix mutations in comparison with different N values.
######################################################################################

###################################
##### Selection and Fixation ######
###################################

#Selection: In case we have a selected mutation, the probability of fixation increases:
#the probability is increased. That is
#the fitness of the selected allele is w = (nm*(1+s) + (N-nm))/N, where s is the selection coefficient
#the probability is therefore: p = nm*(1+s)/(N*w)
#Example: we do a function that includes selection.
calculate.fixation.time.p.selection <- function(initial.p,N,max.generations,s=s) {
  plotf <- array(0,dim=c(max.generations)) 
  p <- initial.p
  g <- 1
  plotf[g] <- p
  while(p > 0 && p < 1 && g < max.generations) {
    p <- p*N*(1+s)/(p*N*(1+s) + (N-p*N))
    Pop <- rbinom(n=N,size=1,prob=p)
    p <- sum(Pop)/N
    g <- g + 1
    plotf[g] <- p
  }
  list(end.p=p,generations=g,plot.freqs=plotf)
}

#Start simulation. Exactly the same than before but replacing the function for the one with selection:
s <- 0.05 #5% more 'progeny' than the rest

par(mfrow=c(1,3))
it <- 1e4
max.g <- 1e4
plotf <- array(0,dim=c(max.g))
plot(x=c(1:max.g),rep(0,max.g),ylim=c(0,1),type="l")
dist.fix <- array(0,dim=c(it))
n.fix <- 0
dist.los <- array(0,dim=c(it))
n.los <- 0

for(i in 1:it) { 
  nm <- 1 #given a mutation, frequency in the initial population
  p <- nm/N
  
  fix.results <- calculate.fixation.time.p.selection(initial.p=p,N=N,max.generations=max.g,s=s)
  plotf <- fix.results$plot.freqs
  g     <- fix.results$generations
  p     <- fix.results$end.p
  
  if(p == 1) {
    lines(x=c(1:g),y=plotf[1:g],type="l",col="black")
    dist.fix[n.fix+1] <- g
    n.fix <- n.fix + 1
  }
  if(p == 0) {
    lines(x=c(1:g),y=plotf[1:g],type="l",col="yellow")
    dist.los[n.los+1] <- g
    n.los <- n.los + 1
  }
}

hist(dist.fix[1:n.fix],breaks=30,main=sprintf("dist.fix: n=%d; mean=%.3f",n.fix,mean(dist.fix[1:n.fix])),freq=FALSE,xlim=c(0,max.g))
abline(v=mean(dist.fix[1:n.fix]),col="black")
n.fix
hist(dist.los[1:n.los],breaks=30,main=sprintf("dist.los: n=%d; mean=%.3f",n.los,mean(dist.los[1:n.los])),freq=FALSE,xlim=c(0,max.g))
abline(v=mean(dist.los[1:n.los]),col="green")
n.los

#proportion of fixed mutations
n.fix/(n.fix+n.los) #1/N under neutrality (total 1/N * it)
#the probability under selection is: (1-exp(-2*s))/(1-exp(-2*N*s))
(1-exp(-2*s))/(1-exp(-2*N*s))

#number of generations to fix mutations
mean(dist.fix[1:n.fix]) #2*N under neutrality

######################################################################################
# EXERCISE 3: Do a simulation using selection (s) positive or negative. Plot the results. 
# Compare the frequency of fixed mutations and the average number of generations that a mutation spent to be fixed with positive or negative selection.
######################################################################################

